@@ -1,9 +1,10 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* vi: set ts=2 sw=2: */
 /* worddic.c
 
    GJITEN : A GTK+/GNOME BASED JAPANESE DICTIONARY
   
-   Copyright (C) 1999 - 2003 Botond Botyanszki <boti@rocketmail.com>
+   Copyright (C) 1999 - 2005 Botond Botyanszki <boti@rocketmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published  by
@@ -53,12 +54,11 @@ static void print_result(gchar *txt2print, int result_offset, gchar *searchstrg)
 static void worddic_close();
 static void worddic_destroy_window();
 
-
 extern GtkWidget *window_kanjidic;
 extern GtkWidget *dialog_preferences;
 extern GjitenConfig conf;
-
 extern GjitenApp *gjitenApp;
+
 WordDic *wordDic = NULL;
 
 int word_matches;
@@ -70,7 +70,6 @@ int dicname_printed;
 int append_to_history = TRUE;
 gpointer current_glist_word = NULL;
 
-
 static GnomeUIInfo file_menu_uiinfo[] = {
   GNOMEUIINFO_MENU_EXIT_ITEM(G_CALLBACK(worddic_destroy_window), NULL),
   GNOMEUIINFO_END
@@ -89,11 +88,11 @@ static GnomeUIInfo edit_menu_uiinfo[] = {
 static GnomeUIInfo tools_menu_uiinfo[] = {
   {
     GNOME_APP_UI_ITEM, N_("KanjiDic"), NULL, kanjidic_create, 
-    NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL
+    NULL, NULL, GNOME_APP_PIXMAP_FILENAME, "kanjidic.png", 0, 0, NULL
   },
   {
     GNOME_APP_UI_ITEM, N_("KanjiPad"), NULL, gjiten_start_kanjipad, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL
+    GNOME_APP_PIXMAP_FILENAME, "kanjipad.png", 0, 0, NULL
   },
   GNOMEUIINFO_END
 };
@@ -248,12 +247,33 @@ static void Verbinit() {
   verbinit_done = TRUE;
 }
 
+static inline void print_matches_in(GjitenDicfile *dicfile) {
+	//Print dicfile name if all dictionaries are selected
+	if ((dicname_printed == FALSE) && (GTK_TOGGLE_BUTTON(wordDic->radiob_searchall)->active)) {
+		gchar *tmpstr, *hl_start_ptr;
+		gint hl_start = 0;
+		gint hl_end = 0; 
+						
+		tmpstr = g_strdup_printf(_("Matches in %s:\n"), dicfile->name);
+		hl_start_ptr = strstr(tmpstr, dicfile->name);
+		hl_start = hl_start_ptr == NULL ? 0 : hl_start_ptr - tmpstr;
+		hl_end = hl_start + strlen(dicfile->name);
 
-void print_verb_inflections(GjitenDicfile *dicfile, gchar *srchstrg) {
+		if (hl_start > 0)
+			gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, tmpstr, hl_start);
+		gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,  
+																						 dicfile->name, -1, "brown_foreground", NULL);
+		gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, tmpstr + hl_end, -1);
+		g_free(tmpstr);
+		dicname_printed = TRUE;
+	}
+}
+
+static void print_verb_inflections(GjitenDicfile *dicfile, gchar *srchstrg) {
 	int srchresp, roff, rlen;
-  int32_t oldrespos, respos;
+  guint32 oldrespos, respos;
   int gjit_search = SRCH_START;
-  unsigned char repstr[1024];
+  gchar repstr[1024];
   GSList *tmp_list_ptr;
   struct vinfl_struct *tmp_vinfl_struct;
   gchar *deinflected, *prevresult;
@@ -261,7 +281,7 @@ void print_verb_inflections(GjitenDicfile *dicfile, gchar *srchstrg) {
 
   tmp_list_ptr = vinfl_list;
   if (vinfl_list == NULL) { 
-    printf("VINFL LIST == NULL\n");
+    //printf("VINFL LIST == NULL\n");
     return;
   }
 
@@ -305,14 +325,7 @@ void print_verb_inflections(GjitenDicfile *dicfile, gchar *srchstrg) {
 					printit = FALSE; // Display only EXACT_MATCHes
 
 				if (printit == TRUE) {
-					//Print dicfile name if all dictionaries are selected
-					if ((dicname_printed == FALSE) && (GTK_TOGGLE_BUTTON(wordDic->radiob_searchall)->active)) {
-						gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, _("Matches in "), -1);
-						gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,  
-																										 dicfile->name, -1, "brown_foreground", NULL);
-						gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, ":\n", -1);
-						dicname_printed = TRUE;
-					}
+					print_matches_in(dicfile);
 					gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,  
 																									 _("Possible inflected verb or adjective: "), 
 																									 -1, "brown_foreground", NULL);
@@ -327,8 +340,7 @@ void print_verb_inflections(GjitenDicfile *dicfile, gchar *srchstrg) {
 				}
       } while (srchresp == SRCH_OK);
     }
-  }    
-  while ((tmp_list_ptr = g_slist_next(tmp_list_ptr)) != NULL);
+  } while ((tmp_list_ptr = g_slist_next(tmp_list_ptr)) != NULL);
   
   g_free(deinflected);
 
@@ -344,25 +356,28 @@ static void print_result(gchar *txt2print, int result_offset, gchar *searchstrg)
   GtkTextMark *linestart;
   GtkTextIter startmatch, endmatch;
 
-  linestart = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(wordDic->text_results_buffer), "linestart", &wordDic->iter, TRUE);
+  linestart = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+																					"linestart", &wordDic->iter, TRUE);
 
   strg_to_roff = (gchar *) g_strndup(txt2print, result_offset);
   strlen_to_roff = g_utf8_strlen(strg_to_roff, -1);
   
   currentchar = txt2print;
 
-  while (g_unichar_isspace(*currentchar) == FALSE) { // find first space
-    if (currentchar - txt2print >= strlen(txt2print)) break;
+	while (!((*currentchar == '[') || (*currentchar == '/'))) { // find end of [KANJI]
+    if ((size_t) (currentchar - txt2print) >= strlen(txt2print)) break;
     currentchar = g_utf8_next_char(currentchar);
   }
-  
-  //print out japanese word
+  currentchar = g_utf8_prev_char(currentchar); // go back to the space
+
+  //print out japanese word == [KANJI]
   if (gjitenApp->conf->bigwords == FALSE) {
-    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, txt2print, currentchar - txt2print); 
+    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+													 &wordDic->iter, txt2print, currentchar - txt2print); 
   }
   else {
-    gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, txt2print, 
-																						 currentchar - txt2print, "largefont", NULL);
+    gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,
+																						 txt2print, currentchar - txt2print, "largefont", NULL);
   }
   
   currentchar = g_utf8_next_char(currentchar);
@@ -372,37 +387,44 @@ static void print_result(gchar *txt2print, int result_offset, gchar *searchstrg)
     while(*(currentchar) != ']') { //find ending ]
       currentchar = g_utf8_next_char(currentchar);
     }
-    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, kana_start, currentchar - kana_start); //print out kana reading
+    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+													 &wordDic->iter, kana_start, currentchar - kana_start); //print out kana reading
     gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, ") ", 2);
     currentchar += 3;
   }
   else {
-    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, " ", 1); // insert space 
+    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+													 &wordDic->iter, " ", 1); // insert space 
     currentchar++;
   }
 
-  while ((currentchar < txt2print + strlen(txt2print)) && (*currentchar != '\n')) {
+  while (currentchar < txt2print + strlen(txt2print)) {
+		if (*currentchar == '\n') break;
     exp_start = currentchar;
-    while ((*currentchar != '/')) { 
+    while (!((*currentchar == '/') || (*currentchar == '\n'))) { 
       currentchar = g_utf8_next_char(currentchar);  
     }
-    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, exp_start, currentchar - exp_start); //print out expression
+		if (*currentchar == '\n') break;
+    gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+													 &wordDic->iter, exp_start, currentchar - exp_start); //print out expression
     gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, "; ", 2);
-    currentchar++;
+		currentchar = g_utf8_next_char(currentchar);  
   }
-  gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, "\n", 1); // insert linebreak
+  gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer),
+												 &wordDic->iter, "\n", 1); // insert linebreak
 
-  //find searchstrg matches in the line we print and highlight it.
+  //find searchstrg matches in the line. we print and highlight it.
   gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &endmatch, linestart);
   
-  while (gtk_text_iter_forward_search(&endmatch, searchstrg, 0, &startmatch, &endmatch, &wordDic->iter) == TRUE) {
+  while (gtk_text_iter_forward_search(&endmatch, searchstrg, 0,
+																			&startmatch, &endmatch, &wordDic->iter) == TRUE) {
     gtk_text_buffer_apply_tag_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), 
 																			"blue_foreground", &startmatch, &endmatch);
   }
 } 
 
 
-static void search_in_dicfile(GjitenDicfile *dicfile, unsigned char *srchstrg) {
+static void search_in_dicfile(GjitenDicfile *dicfile, gchar *srchstrg) {
   gint srchresp, roff, rlen;
   gchar repstr[1024];
   guint32 respos, oldrespos;
@@ -422,12 +444,10 @@ static void search_in_dicfile(GjitenDicfile *dicfile, unsigned char *srchstrg) {
       jpsrch = TRUE;
       break;
     }
-  }
-  while ((currchar = g_utf8_find_next_char(currchar, srchstrg + strlen(srchstrg))) != NULL);
+  } while ((currchar = g_utf8_find_next_char(currchar, srchstrg + strlen(srchstrg))) != NULL);
   
   // Verb deinfelction
-  if (GTK_TOGGLE_BUTTON(wordDic->checkb_verb)->active) print_verb_inflections(dicfile, srchstrg);
-  
+  if (gjitenApp->conf->verb_deinflection == TRUE) print_verb_inflections(dicfile, srchstrg);
 
   if (jpsrch == TRUE) {
     if (GTK_TOGGLE_BUTTON(wordDic->radiob_jpexact)->active) match_criteria = EXACT_MATCH;
@@ -441,7 +461,6 @@ static void search_in_dicfile(GjitenDicfile *dicfile, unsigned char *srchstrg) {
     if (GTK_TOGGLE_BUTTON(wordDic->radiob_partial)->active) match_criteria = ANY_MATCH;
   }
 
-
   oldrespos = srchpos = 0;
   
   do { // search loop
@@ -518,21 +537,13 @@ static void search_in_dicfile(GjitenDicfile *dicfile, unsigned char *srchstrg) {
       printf("criteria: %d\n", match_criteria);
       */
 
-			//Print dicfile name if all dictionaries are selected
-      if ((dicname_printed == FALSE) && (GTK_TOGGLE_BUTTON(wordDic->radiob_searchall)->active)) {
-				gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, _("Matches in "), -1);
-				gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,  
-																								 dicfile->name, -1, "brown_foreground", NULL);
-				gtk_text_buffer_insert(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, ":\n", -1);
-				dicname_printed = TRUE;
-      }
+      print_matches_in(dicfile);
       print_result(repstr, roff, srchstrg);
       //get_kanji_and_reading(repstr); FIXME
       word_matches++;
     }
     if ((gjitenApp->conf->searchlimit_enabled == TRUE) && (word_matches >= gjitenApp->conf->maxwordmatches)) {
-      gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter,  
-																							 _("Results truncated"), -1, "red_foreground", NULL);
+      gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(wordDic->text_results_buffer), &wordDic->iter, _("Results truncated"), -1, "red_foreground", NULL);
       return;
     }
   } while (srchresp == SRCH_OK);
@@ -572,7 +583,7 @@ int lower_search_option() {
   return FALSE;
 }
 
-void worddic_hira_kata_search(GjitenDicfile *dicfile, gchar *srchstrg) {
+static void worddic_hira_kata_search(GjitenDicfile *dicfile, gchar *srchstrg) {
 	gchar *hirakata;
 	if (gjitenApp->conf->search_kata_on_hira) {
 		if (isKatakanaString(srchstrg) == TRUE) {
@@ -590,7 +601,7 @@ void worddic_hira_kata_search(GjitenDicfile *dicfile, gchar *srchstrg) {
 	}
 }
 
-void worddic_search(unsigned char *srchstrg) {
+static void worddic_search(gchar *srchstrg) {
   gchar appbarmsg[50];
   int truncated;
 	GjitenDicfile *dicfile;
@@ -599,7 +610,7 @@ void worddic_search(unsigned char *srchstrg) {
   word_matches = 0;
 
 	if (gjitenApp->conf->dicfile_list == NULL) {
-    sprintf(appbarmsg,_("No dicfiles specified! Set your preferences first."));
+    snprintf(appbarmsg, 50, _("No dicfiles specified! Set your preferences first."));
     gnome_appbar_set_status(GNOME_APPBAR(wordDic->appbar_mainwin),appbarmsg);
     return;
   } 
@@ -637,8 +648,8 @@ void worddic_search(unsigned char *srchstrg) {
   }
 
   if (word_matches) {
-    if (truncated) sprintf(appbarmsg, _("Matches found (truncated): %d"), word_matches);
-    else sprintf(appbarmsg, _("Matches found: %d"), word_matches);
+    if (truncated) snprintf(appbarmsg, 50, _("Matches found (truncated): %d"), word_matches);
+    else snprintf(appbarmsg, 50, _("Matches found: %d"), word_matches);
     gnome_appbar_set_status(GNOME_APPBAR(wordDic->appbar_mainwin), appbarmsg);
   }
   else gnome_appbar_set_status(GNOME_APPBAR(wordDic->appbar_mainwin), _("No match found!"));
@@ -647,6 +658,9 @@ void worddic_search(unsigned char *srchstrg) {
 void on_text_entered() {
   static gchar *new_entry_text = NULL;
 
+	gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(wordDic->text_results_view), GTK_TEXT_WINDOW_TEXT), wordDic->regular_cursor);
+	wordDic->is_cursor_regular = TRUE;
+
   new_entry_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wordDic->combo_entry)->entry)));
   if (g_utf8_validate(new_entry_text, -1, NULL) == FALSE) {
     gjiten_print_error_and_wait(_("Invalid input: non-utf8\n")); 
@@ -720,8 +734,8 @@ static void shade_worddic_widgets() {
   if ((wordDic->menu_selectdic != NULL) && (wordDic->radiob_searchdic != NULL)) 
     gtk_widget_set_sensitive(wordDic->menu_selectdic, GTK_TOGGLE_BUTTON(wordDic->radiob_searchdic)->active);
 
-  if (wordDic->checkb_autoadjust != NULL) gjitenApp->conf->autoadjust_enabled = (GTK_TOGGLE_BUTTON(wordDic->checkb_autoadjust)->active);
-  if (wordDic->checkb_verb != NULL) gjitenApp->conf->deinflection_enabled = (GTK_TOGGLE_BUTTON(wordDic->checkb_verb)->active);  
+  if (wordDic->checkb_autoadjust != NULL)
+		gjitenApp->conf->autoadjust_enabled = (GTK_TOGGLE_BUTTON(wordDic->checkb_autoadjust)->active);
 }
 
 
@@ -754,11 +768,9 @@ static void worddic_init_history() {
 }
 
 static void worddic_destroy_window() {
-
 	if ((wordDic != NULL) && (GTK_IS_WIDGET(wordDic->window))) {
 		gtk_widget_destroy(wordDic->window);
 	}
-
 }
 
 static void worddic_close() {
@@ -846,6 +858,63 @@ void worddic_apply_fonts() {
 
 }
 
+
+/*
+ * Update the cursor image if the pointer is above a kanji. 
+ */
+static gboolean result_view_motion(GtkWidget *text_view, GdkEventMotion *event) {
+  gint x, y;
+  GtkTextIter mouse_iter;
+	gunichar kanji;
+	gint trailing;
+
+  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), 
+																				GTK_TEXT_WINDOW_WIDGET,
+																				event->x, event->y, &x, &y);
+
+  gtk_text_view_get_iter_at_position(GTK_TEXT_VIEW(text_view), &mouse_iter, &trailing, x , y);
+	kanji = gtk_text_iter_get_char(&mouse_iter);
+
+	// Change the cursor if necessary
+	if ((isKanjiChar(kanji) == TRUE)) {
+		gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_TEXT), wordDic->selection_cursor);
+		wordDic->is_cursor_regular = FALSE;
+	}
+	else if (wordDic->is_cursor_regular == FALSE) {
+		gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_TEXT), wordDic->regular_cursor);
+		wordDic->is_cursor_regular = TRUE;
+	}
+	gdk_window_get_pointer(text_view->window, NULL, NULL, NULL);
+
+  return FALSE;
+}
+
+
+
+static gboolean kanji_clicked(GtkWidget *text_view, GdkEventButton *event, gpointer user_data) {
+	GtkTextIter mouse_iter;
+	gint x, y;
+	gint trailing;
+	gunichar kanji;
+
+  if (event->button != 1) return FALSE;
+
+  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (text_view), 
+																				GTK_TEXT_WINDOW_WIDGET,
+																				event->x, event->y, &x, &y);
+
+  gtk_text_view_get_iter_at_position(GTK_TEXT_VIEW(text_view), &mouse_iter, &trailing, x, y);
+	kanji = gtk_text_iter_get_char(&mouse_iter);
+	if ((kanji != 0xFFFC) && (kanji != 0) && (isKanjiChar(kanji) == TRUE)) {
+		kanjidic_create();
+		kanji_selected(kanji);
+	}
+
+
+  return FALSE;
+}
+
+
 WordDic *worddic_create() {
   GtkWidget *vbox_main;
   GtkWidget *dock_main;
@@ -876,18 +945,22 @@ WordDic *worddic_create() {
   GtkWidget *scrolledwin_results;
   GtkObject *spinb_searchlimit_adj;
   GtkWidget *tmpimage;
-
+  GdkPixbuf *cursor_pixbuf;
 
 	if (wordDic == NULL) {
 		wordDic = g_new0(WordDic, 1);
 		gjitenApp->worddic = wordDic;
 	}
 	else {
-		gtk_widget_hide(wordDic->window);
-		gtk_widget_show(wordDic->window);
+		gtk_window_present(GTK_WINDOW(wordDic->window));
 		return wordDic;
 	}
 
+  cursor_pixbuf = gdk_pixbuf_new_from_file(PIXMAPDIR"/left_ptr_question.png", NULL);
+  wordDic->selection_cursor = gdk_cursor_new_from_pixbuf(gdk_display_get_default(), cursor_pixbuf, 0, 0);
+  wordDic->regular_cursor = gdk_cursor_new(GDK_XTERM);
+	wordDic->is_cursor_regular = TRUE;
+
   wordDic->spinb_searchlimit = NULL;
   wordDic->radiob_searchdic = NULL;
   wordDic->checkb_autoadjust = NULL;
@@ -896,7 +969,7 @@ WordDic *worddic_create() {
 	worddic_init_history();
   Verbinit(); //FIXME: On demand
 
-  wordDic->window = gnome_app_new("gjiten", "Gjiten - WordDic");
+  wordDic->window = gnome_app_new("gjiten", _("Gjiten - WordDic"));
   GTK_WIDGET_SET_FLAGS(wordDic->window, GTK_CAN_DEFAULT);
   g_signal_connect(G_OBJECT(wordDic->window), "destroy", G_CALLBACK(worddic_close), NULL);
   gtk_window_set_default_size(GTK_WINDOW(wordDic->window), 500, 500);
@@ -913,33 +986,33 @@ WordDic *worddic_create() {
     gnome_app_set_toolbar(GNOME_APP(wordDic->window), GTK_TOOLBAR(toolbar));
     
     button_exit = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar), GTK_STOCK_CLOSE,
-																					 _("Close"), "Close", NULL, NULL, -1);
+																					 _("Close Gjiten"), "Close", NULL, NULL, -1);
 		g_signal_connect_swapped(G_OBJECT(button_exit), "clicked", 
 														 G_CALLBACK(gtk_widget_destroy), wordDic->window);
 
     wordDic->button_back = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar), GTK_STOCK_GO_BACK,
-																										_("Back"), "Back", 
+																										_("Previous search result"), "Back", 
 																										on_back_clicked, NULL, -1);
     gtk_widget_set_sensitive(wordDic->button_back, FALSE);
 
     wordDic->button_forward = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar), GTK_STOCK_GO_FORWARD,
-																											 _("Forward"), "Forward", 
+																											 _("Next search result"), "Forward", 
 																											 on_forward_clicked, NULL, -1);
     gtk_widget_set_sensitive(wordDic->button_forward, FALSE);
 
 
     tmpimage = gtk_image_new_from_file(PIXMAPDIR"/kanjidic.png");
     button_kanjidic = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), _("KanjiDic"),
-																							_("KanjiDic"), "KanjiDic", tmpimage,
+																							_("Launch KanjiDic"), "KanjiDic", tmpimage,
 																							G_CALLBACK(kanjidic_create), NULL);
 
     tmpimage = gtk_image_new_from_file(PIXMAPDIR"/kanjipad.png");
     button_kanjipad = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), _("KanjiPad"),
-																							_("KanjiPad"), "KanjiPad", tmpimage,
+																							_("Launch KanjiPad"), "KanjiPad", tmpimage,
 																							G_CALLBACK(gjiten_start_kanjipad), NULL);
 
     button_srch = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar), GTK_STOCK_FIND,
-																					 _("Search"), "Search", 
+																					 _("Search for entered expression"), "Search", 
 																					 on_text_entered, NULL, -1);
 
 		/*
@@ -976,8 +1049,6 @@ WordDic *worddic_create() {
     gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
 
   }    
-    
-
 
   vbox_main = gtk_vbox_new(FALSE, 0);
   gtk_widget_show(vbox_main);
@@ -1016,14 +1087,6 @@ WordDic *worddic_create() {
   gtk_widget_show(wordDic->radiob_any);
   gtk_box_pack_start(GTK_BOX(vbox_japopt), wordDic->radiob_any, FALSE, FALSE, 0);
 
-  wordDic->checkb_verb = gtk_check_button_new_with_mnemonic(_("_Verb Deinflection"));
-  gtk_widget_show(wordDic->checkb_verb);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wordDic->checkb_verb), TRUE);
-  gtk_box_pack_start(GTK_BOX(vbox_japopt), wordDic->checkb_verb, FALSE, FALSE, 0);
-  g_signal_connect(G_OBJECT(wordDic->checkb_verb), "toggled", 
-									 G_CALLBACK(shade_worddic_widgets), NULL);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wordDic->checkb_verb), gjitenApp->conf->deinflection_enabled);
-  
   frame_engopt = gtk_frame_new(_("English Search Options: "));
   gtk_widget_show(frame_engopt);
   gtk_box_pack_start(GTK_BOX(wordDic->hbox_options), frame_engopt, TRUE, TRUE, 0);
@@ -1081,7 +1144,7 @@ WordDic *worddic_create() {
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
 									 (GtkAttachOptions)(0), 0, 0);
   g_signal_connect_swapped(G_OBJECT(wordDic->radiob_searchall), "clicked", 
-			     G_CALLBACK(shade_worddic_widgets), NULL);
+													 G_CALLBACK(shade_worddic_widgets), NULL);
 
 
   wordDic->checkb_autoadjust = gtk_check_button_new_with_mnemonic(_("A_uto Adjust Options"));
@@ -1111,9 +1174,9 @@ WordDic *worddic_create() {
   wordDic->spinb_searchlimit = gtk_spin_button_new(GTK_ADJUSTMENT(spinb_searchlimit_adj), 1, 0);
   gtk_widget_show(wordDic->spinb_searchlimit);
   gtk_box_pack_start(GTK_BOX(hbox_searchlimit), wordDic->spinb_searchlimit, FALSE, FALSE, 0);
-  gtk_widget_set_sensitive(wordDic->spinb_searchlimit,(GTK_TOGGLE_BUTTON(wordDic->checkb_searchlimit)->active));
+  gtk_widget_set_sensitive(wordDic->spinb_searchlimit, (GTK_TOGGLE_BUTTON(wordDic->checkb_searchlimit)->active));
   g_signal_connect(G_OBJECT(spinb_searchlimit_adj), "value_changed", 
-		      G_CALLBACK(get_searchlimit), NULL);
+									 G_CALLBACK(get_searchlimit), NULL);
   
 
   hbox_entry = gtk_hbox_new(FALSE, 0);   
@@ -1133,7 +1196,8 @@ WordDic *worddic_create() {
   gtk_box_pack_start(GTK_BOX(hbox_entry), wordDic->combo_entry, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(GTK_COMBO(wordDic->combo_entry)->entry), 
 									 "activate", G_CALLBACK(on_text_entered), NULL);
-  g_signal_connect(G_OBJECT(wordDic->window), "key_press_event", G_CALLBACK(set_focus_on_entry), GTK_COMBO(wordDic->combo_entry)->entry);
+  g_signal_connect(G_OBJECT(wordDic->window), "key_press_event",
+									 G_CALLBACK(set_focus_on_entry), GTK_COMBO(wordDic->combo_entry)->entry);
 
   gtk_combo_disable_activate(GTK_COMBO(wordDic->combo_entry));
   gtk_combo_set_case_sensitive(GTK_COMBO(wordDic->combo_entry), TRUE);
@@ -1149,7 +1213,7 @@ WordDic *worddic_create() {
   button_search = gtk_button_new_with_label(_("Search"));
   gtk_widget_show(button_search);
   gtk_box_pack_start(GTK_BOX(hbox_entry), button_search, FALSE, FALSE, 7);
-  g_signal_connect(G_OBJECT(button_search), "clicked",G_CALLBACK(on_text_entered), NULL);
+  g_signal_connect(G_OBJECT(button_search), "clicked", G_CALLBACK(on_text_entered), NULL);
 
   button_clear = gtk_button_new_with_mnemonic(_("_Clear"));
   gtk_widget_show(button_clear);
@@ -1173,7 +1237,12 @@ WordDic *worddic_create() {
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(wordDic->text_results_view), GTK_WRAP_WORD);
 
   gtk_widget_show(wordDic->text_results_view);
-	gtk_text_view_set_editable(GTK_TEXT_VIEW(wordDic->text_results_view), FALSE);
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(wordDic->text_results_view), FALSE);
+	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(wordDic->text_results_view), FALSE);
+
+  // enable clickable kanji
+  g_signal_connect(G_OBJECT(wordDic->text_results_view), "button-release-event", G_CALLBACK(kanji_clicked), NULL);
+  g_signal_connect(G_OBJECT(wordDic->text_results_view), "motion-notify-event", G_CALLBACK(result_view_motion), NULL);
   
   //set up fonts and tags
   gtk_text_buffer_create_tag(wordDic->text_results_buffer, "blue_foreground", "foreground", "blue", NULL);  
